@@ -60,7 +60,7 @@ import numpy
 import scipy.sparse
 
 from gensim import interfaces, utils, matutils
-from gensim._six.moves import map as imap, xrange, zip as izip
+from six.moves import map as imap, xrange, zip as izip
 
 
 logger = logging.getLogger('gensim.similarities.docsim')
