@@ -89,6 +89,11 @@ class TestWord2VecModel(unittest.TestCase):
         binary_model_with_vocab = word2vec.Word2Vec.load_word2vec_format(testfile(), testvocab, binary=True)
         self.assertEqual(model.vocab['human'].count, binary_model_with_vocab.vocab['human'].count)
 
+    def test_zero_workers_mode(self):
+        model = word2vec.Word2Vec(sentences, min_count=1)
+        model0 = word2vec.Word2Vec(sentences, min_count=1, workers=0)
+        self.models_equal(model,model0)
+
     def testLargeMmap(self):
         """Test storing/loading the entire model."""
         model = word2vec.Word2Vec(sentences, min_count=1)
@@ -110,8 +115,7 @@ class TestWord2VecModel(unittest.TestCase):
         model.build_vocab(corpus)
         self.assertTrue(len(model.vocab) == 6981)
         # with min_count=1, we're not throwing away anything, so make sure the word counts add up to be the entire corpus
-        self.assertEqual(sum(v.count for v in model.vocab.values()),
-                         total_words)
+        self.assertEqual(sum(v.count for v in model.vocab.values()), total_words)
         # make sure the binary codes are correct
         numpy.allclose(model.vocab['the'].code, [1, 1, 0, 0])
 
@@ -143,12 +147,32 @@ class TestWord2VecModel(unittest.TestCase):
         # test querying for "most similar" by vector
         graph_vector = model.syn0norm[model.vocab['graph'].index]
         sims2 = model.most_similar(positive=[graph_vector], topn=11)
-        self.assertEqual(sims, sims2[1:])  # ignore first element of sims2, which is 'graph' itself
+        sims2 = [(w, sim) for w, sim in sims2 if w != 'graph']  # ignore 'graph' itself
+        self.assertEqual(sims, sims2)
 
         # build vocab and train in one step; must be the same as above
         model2 = word2vec.Word2Vec(sentences, size=2, min_count=1)
         self.models_equal(model, model2)
 
+
+    def testLocking(self):
+        """Test word2vec training doesn't change locked vectors."""
+        corpus = LeeCorpus()
+        # build vocabulary, don't train yet
+        for sg in range(2):  # test both cbow and sg
+            model = word2vec.Word2Vec(size=4, hs=1, negative=5, min_count=1, sg=sg, window=5)
+            model.build_vocab(corpus)
+
+            # remember two vectors
+            locked0 = numpy.copy(model.syn0[0])
+            unlocked1 = numpy.copy(model.syn0[1])
+            # lock the vector in slot 0 against change
+            model.syn0_lockf[0] = 0.0
+
+            model.train(corpus)
+            self.assertFalse((unlocked1==model.syn0[1]).all())  # unlocked vector should vary
+            self.assertTrue((locked0==model.syn0[0]).all())     # locked vector should not vary
+
     def testTrainingCbow(self):
         """Test CBOW word2vec training."""
         # to test training, make the corpus larger by repeating its sentences over and over
@@ -165,7 +189,8 @@ class TestWord2VecModel(unittest.TestCase):
         # test querying for "most similar" by vector
         graph_vector = model.syn0norm[model.vocab['graph'].index]
         sims2 = model.most_similar(positive=[graph_vector], topn=11)
-        self.assertEqual(sims, sims2[1:])  # ignore first element of sims2, which is 'graph' itself
+        sims2 = [(w, sim) for w, sim in sims2 if w != 'graph']  # ignore 'graph' itself
+        self.assertEqual(sims, sims2)
 
         # build vocab and train in one step; must be the same as above
         model2 = word2vec.Word2Vec(sentences, size=2, min_count=1, sg=0)
@@ -187,7 +212,8 @@ class TestWord2VecModel(unittest.TestCase):
         # test querying for "most similar" by vector
         graph_vector = model.syn0norm[model.vocab['graph'].index]
         sims2 = model.most_similar(positive=[graph_vector], topn=11)
-        self.assertEqual(sims, sims2[1:])  # ignore first element of sims2, which is 'graph' itself
+        sims2 = [(w, sim) for w, sim in sims2 if w != 'graph']  # ignore 'graph' itself
+        self.assertEqual(sims, sims2)
 
         # build vocab and train in one step; must be the same as above
         model2 = word2vec.Word2Vec(sentences, size=2, min_count=1, hs=0, negative=2)
@@ -209,7 +235,8 @@ class TestWord2VecModel(unittest.TestCase):
         # test querying for "most similar" by vector
         graph_vector = model.syn0norm[model.vocab['graph'].index]
         sims2 = model.most_similar(positive=[graph_vector], topn=11)
-        self.assertEqual(sims, sims2[1:])  # ignore first element of sims2, which is 'graph' itself
+        sims2 = [(w, sim) for w, sim in sims2 if w != 'graph']  # ignore 'graph' itself
+        self.assertEqual(sims, sims2)
 
         # build vocab and train in one step; must be the same as above
         model2 = word2vec.Word2Vec(sentences, size=2, min_count=1, sg=0, hs=0, negative=2)
